/**
Silicium628
J'écris cette bibliothèque en suivant le datasheet d'ATMEL point par point. (p:170 et suivante datasheet ATmega8A)
* 
On trouve ce genre de "library" (wire.h) dans la logithèque Arduino mais une étude approfondie m'a permis de constater
que cette librairy nécessite par ses dépendances (include) l'utilisation de plusieurs dizaines d'autres librairy !!

Voyez plutôt :
(Arduino.h, binary.h, HardwareSerial.h, HardwareSerial.cpp, I2C, new.h, new.cpp, pins_arduino.h, Print.cpp, Print.h,
Printable.h,  Stream.cpp, Stream.h, TEA5767.cpp, TEA5767.h, timeout.h, twi.c, twi.h, WCharacter.h, Wire.cpp, Wire.h
WString.cpp, WString.h ...) et j'en oublie, je n'ai pas tout exploré,  STOP !

Le TEA5767 nécessite qu'on lui envoie 5 octets par une liaison I2C pour fixer sa fréquence de réception, je pense que ça ne justifie pas
d'écrire plusieurs milliers de lignes de code. STOP !!

Et en fait la trentaine de lignes qui suivent y parviennent parfaitement.

20.4.5 - A transmission basically consists of a START condition, a SLA+R/W, one or more data packets and a STOP condition. ONE OR MORE !

 
**/


#include <avr/io.h>
#include <stdint.h> 

#define TW_START     0x08   //TWI start
#define TW_REP_START 0x10   //TWI repeat start
#define MT_SLA_ACK	 0x18   //slave ACK has been received  //  voir datasheet ATmega8A (tableau p:179)
#define MT_DATA_ACK	 0x28   //master ACK has been received
#define READ_ONLY    0x00   //Read only pointer register mode


void out_stop()
{
  TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWSTO) ;
}


int i2c_write_n_bytes(uint8_t SLA_addr, uint8_t data[], uint8_t nb_octets) // Passage du tableau en paramètre (on n'écrit pas l'étoile dans ce cas d'un tableau)
{
  uint8_t i;
  // SLA_W = adresse de l'esclave (8 bits) - p:166 20.2.3

// voir tableau p:176
  
// start  
  TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN); // Send START condition
  while (!(TWCR & (1<<TWINT))) {;}// Wait for TWINT Flag set. This indicates that the START condition has been transmitted
  if ((TWSR & 0xF8) != TW_START) {out_stop(); return 1; } //Check value of TWI Status Register. Mask prescaler bits. If status different from START go to ERROR

// adresse  
  TWDR = (SLA_addr << 1) & 0b11111110; // charge les 7 bits d'adresse dans TWDR (8bits), et ajoute un 0 à droite (le bit R/W, ici =0 -> write).
  TWCR = (1<<TWINT) | (1<<TWEN); //Clear TWINT bit in TWCR to start transmission of address
  while ((TWCR & (1<<TWINT))==0) {;}//Wait for TWINT Flag set. This indicates that the SLA+W has been transmitted, and ACK(Acknowledge)/NACK(Non Acknowledge) has been received.
  if ((TWSR & 0xF8) != MT_SLA_ACK) {out_stop(); return 2; } // Check value of TWI Status Register. Mask prescaler bits. If status different from MT_SLA_ACK go to ERROR

 
// data
  for(i = 0; i<nb_octets; i++)
  {
    TWDR = data[i]; //Load DATA into TWDR Register.
    TWCR = (1<<TWINT) | (1<<TWEN); // Clear TWINT bit in TWCR to start transmission of data
    while (!(TWCR & (1<<TWINT))) {;} //Wait for TWINT Flag set. This indicates that the DATA has been transmitted, and ACK/NACK has been received.
    if ((TWSR & 0xF8) != MT_DATA_ACK)  { out_stop(); return 3;  } // Check value of TWI Status Register. Mask prescaler bits. If status different from MT_DATA_ACK go to ERROR
  }

// stop 
  out_stop(); // Transmit STOP condition
  return 0;
}




